<?php

/**
 * @file
 * The administration interface.
 */

/**
 * XLIFF export API for given nodes.
 */
function xliff_bulk_export($nids, $bilingual = FALSE) {
  $error_count = 0;
  $tar = array();
  foreach ($nids as $nid) {
    if ($bilingual) {
      $output = xliff_bilingual_export($nid, $message);
    }
    else {
      $output = xliff_export($nid);
    }
    if (empty($output)) {
      watchdog('xliff_bulkexport', 'Failed to export <a href="/node/@node">node @node</a>.', array('@node' => $nid), WATCHDOG_ERROR);
      $error_count++;
    }
    else {
      $node = db_fetch_object(db_query("SELECT n.language FROM {node} n WHERE n.nid = %d", $nid));
      $langcode = (!empty($node->language)) ? $node->language : language_default('language');
      $tar["node-$nid-$langcode.xlf"] = $output;
    }
  }

  $success_count = count($nids) - $error_count;
  if ($success_count) {
    drupal_set_message(t('%node_count exported.', array('%node_count' => format_plural($success_count, '1 node', '@count nodes'))));
  }
  if ($error_count) {
    $dblog_link = '';
    if (user_access('access site reports')) {
      $dblog_link = ' '. l(t('View report'), 'admin/reports/dblog');
    }
    drupal_set_message(t('Failed to export %node_count.', array('%node_count' => format_plural($error_count, '1 node', '@count nodes'))) . $dblog_link, 'error');
  }

  if (empty($tar)) {
    return;
  }

  $tar = xliff_bulk_tar_create($tar);

  if (function_exists('gzencode')) {
    $header = 'Content-type: application/x-gzip';
    $filename = 'xliff-bulk-export.tgz';
  }
  else {
    $header = 'Content-type: application/x-tar';
    $filename = 'xliff-bulk-export.tar';
  }

  // Clear out output buffer to remove any garbage from tar output.
  if (ob_get_level()) {
    ob_end_clean();
  }

  drupal_set_header($header);
  drupal_set_header('Content-Disposition: attachment; filename="'. $filename .'"');
  print $tar;
  exit;
}

/**
 * Menu callback; Admin form for xliff bulk export.
 */
function xliff_bulk_export_form(&$form_state) {
  $langcode = isset($_SESSION['xliff_bulk']['langcode']) ? $_SESSION['xliff_bulk']['langcode'] : language_default('language');
  $node_type = isset($_SESSION['xliff_bulk']['node_type']) ? $_SESSION['xliff_bulk']['node_type'] : 'page'; //@todo: fix default.
  $translation_status = isset($_SESSION['xliff_bulk']['translation_status']) ? $_SESSION['xliff_bulk']['translation_status'] : 0;
  $node_status = isset($_SESSION['xliff_bulk']['node_status']) ? $_SESSION['xliff_bulk']['node_status'] : -1;

  $form['filter'] = xliff_bulk_export_filter($langcode, $node_type, $translation_status, $node_status);
  $form['admin'] = xliff_bulk_export_filtered_form($langcode, $node_type, $translation_status, $node_status);
  $form['#submit'][] = 'xliff_bulk_export_form_submit';
  $form['#submit'][] = 'xliff_bulk_export_submit';
  return $form;
}

/**
 * Submit handler for the xliff bulk export form.
 */
function xliff_bulk_export_form_submit($form, &$form_state) {
  switch ($form_state['clicked_button']['#id']) {
    case 'edit-filter':
    case 'edit-submit':
    case 'edit-submit-all':
      $_SESSION['xliff_bulk']['node_type'] = $form_state['values']['node_type'];
      $_SESSION['xliff_bulk']['translation_status'] = $form_state['values']['translation_status'];
      $_SESSION['xliff_bulk']['node_status'] = $form_state['values']['node_status'];
      $_SESSION['xliff_bulk']['langcode'] = $form_state['values']['langcode'];
      break;
  }
}

/**
 * Submit handler for the xliff bulk export.
 */
function xliff_bulk_export_submit($form, &$form_state) {
  $bilingual = $form_state['values']['bilingual'];
  switch ($form_state['clicked_button']['#id']) {
    case 'edit-submit':
      $nids = array_filter($form_state['values']['nodes']);
      xliff_bulk_export($nids, $bilingual);
      break;

    case 'edit-submit-all':
      $nids = array();
      $node_type = $form_state['values']['node_type'];
      $langcode = $form_state['values']['langcode'];
      $status = $form_state['values']['node_status'];
      $sql = "SELECT n.nid FROM {node} n WHERE n.type = '%s' AND n.language = '%s'";
      if (in_array($status, array(0, 1))) {
        $sql .= " AND n.status = %d";
        $result = db_query($sql, $node_type, $langcode, $status);
      }
      else {
        $result = db_query($sql, $node_type, $langcode);
      }
      while ($node = db_fetch_object($result)) {
        $nids[$node->nid] = $node->nid;
      }
      xliff_bulk_export($nids, $bilingual);
      break;
  }
}

/**
 * Node type filter.
 */
function xliff_bulk_export_filter($langcode, $node_type, $translation_status, $node_status) {
  $form['langcode'] = array(
    '#type' => 'select',
    '#title' => t('Language'),
    '#description' => t('Select the language to display.'),
    '#options' => array('' => t('Language neutral')) + locale_language_list('name', TRUE),
    '#default_value' => $langcode,
    '#prefix' => '<div id="language-filter">',
    '#suffix' => '</div>',
  );

  $form['node_type'] = array(
    '#type' => 'select',
    '#title' => t('Content type'),
    '#description' => t('Select the content types to display.'),
    '#options' => node_get_types('names'),
    '#default_value' => $node_type,
    '#prefix' => '<div id="node-type-filter">',
    '#suffix' => '</div>',
  );

  $status_options = array(
    -1 => '<'. t('Any') .'>',
    1 => t('published'),
    0 => t('not published'),
  );
  $form['node_status'] = array(
    '#type' => 'select',
    '#title' => t('Status'),
    '#description' => t('Select the content status.'),
    '#options' => $status_options,
    '#default_value' => $node_status,
    '#prefix' => '<div id="node-status-filter">',
    '#suffix' => '</div>',
  );

  // Include translation states if we have this module enabled
  /*if (module_exists('translation')) {
    $filters['status']['options'] += array(
      'translate-0' => t('Up to date translation'),
      'translate-1' => t('Outdated translation'),
    );
  }*/

  $form['translation_status'] = array(
    '#type' => 'hidden',
    '#title' => t('Translation status'),
    '#description' => t('Select the translation status.'),
    '#default_value' => $translation_status,
    '#prefix' => '<div id="translation-status-filter">',
    '#suffix' => '</div>',
  );

  $form['filter'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
  );

  $form['#theme'] = 'xliff_bulk_export_filter';
  return $form;
}

/**
 * Theme function for the export filter form.
 *
 * @ingroup themeable
 */
function theme_xliff_bulk_export_filter($form) {
  $output = drupal_render($form);
  return $output;
}

/**
 * Form for bulk_export.
 *
 * @param $langcode
 *   filter by language code
 * @param $node_type
 *   filter by node type
 * @param $translation_status
 *   ?
 */
function xliff_bulk_export_filtered_form($langcode, $node_type, $translation_status, $node_status) {
  $header = array(
    array('data' => t('Title'), 'field' => 'title'),
    array('data' => t('Type'), 'field' => 'type'),
    array('data' => t('Language'), 'field' => 'language'),
    array('data' => t('Status'), 'field' => 'status'),
    array('data' => t('Updated'), 'field' => 'changed', 'sort' => 'desc'),
  );

  $sql = "SELECT n.nid, n.title, n.type, n.language, n.status, n.changed FROM {node} n WHERE n.type = '%s' AND n.language = '%s'";
  if (in_array($node_status, array(0, 1))) {
    $sql .= " AND n.status = %d";
    $sql .= tablesort_sql($header);
    $result = pager_query($sql, 50, 0, NULL, $node_type, $langcode, $node_status);

  }
  else {
    $sql .= tablesort_sql($header);
    $result = pager_query($sql, 50, 0, NULL, $node_type, $langcode);
  }

  $form['strings']['#tree'] = TRUE;
  $form['#cache'] = TRUE;

  $form['header'] = array(
    '#type' => 'value',
    '#value' => $header,
  );

  $nodes = array();
  while ($node = db_fetch_object($result)) {
    $nodes[$node->nid] = '';
    $form['node'][$node->nid] = _xliff_bulk_export_node_list_row($node);
  }
  $form['nodes'] = array(
    '#type' => 'checkboxes',
     '#options' => $nodes,
  );

  $form['bilingual'] = array(
    '#type' => 'checkbox',
    '#title' => t('Bilingual mode'),
    '#default_value' => 0,
    '#description' => t('The XLIFF Export will be done in bilingual mode. Can be used on translations only. There is no guarantee that this export will work as node translations may have different html format, therefore the source and target strings could not be paired.'),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Export selected'),
  );

  $form['submit_all'] = array(
    '#type' => 'submit',
    '#value' => t('Export all'),
  );

  // @todo: This should be automatic somehow.
  $form['help'] = array(
    '#type' => 'markup',
    '#value' => '<br />'. t('After file save please reload the page to see the result count and potential errors.') .'<br />',
  );
  $form['pager'] = array('#value' => theme('pager', NULL, 50, 0));
  $form['#theme'] = 'xliff_bulk_export_node_list';
  return $form;
}

/**
 * One row of the node list.
 *
 * @param $node
 *   node
 * @return
 *   table row
 */
function _xliff_bulk_export_node_list_row($node) {
  $form['title'] = array(
    '#type' => 'markup',
    '#value' => l($node->title, 'node/'. $node->nid),
  );
  $node_types = node_get_types('names');
  $form['type'] = array(
    '#type' => 'markup',
    '#value' => check_plain($node_types[$node->type]),
  );
  $languages = language_list();
  $form['language'] = array(
    '#type' => 'markup',
    '#value' => empty($node->language) ? t('Language neutral') : t($languages[$node->language]->name),
  );
  $status_options = array(0 => t('not published'), 1 => t('published'));
  $form['status'] = array(
    '#type' => 'markup',
    '#value' => $status_options[$node->status],
  );
  $form['changed'] = array(
    '#type' => 'markup',
    '#value' => format_date($node->changed, 'short'),
  );

  return $form;
}

/**
 * Theme function for the node list.
  *
 * @ingroup themeable
 */
function theme_xliff_bulk_export_node_list($form) {
  $rows = array();
  $has_posts = isset($form['node']);
  $select_header = $has_posts ? theme('table_select_header_cell') : '';
  $header = array('-1' => $select_header) + $form['header']['#value'];

  if ($has_posts) {
    foreach (element_children($form['node']) as $nid) {
      // Build the table row.
      $row = array(
        array('data' => drupal_render($form['nodes'][$nid])),
        array('data' => drupal_render($form['node'][$nid]['title']), 'class' => 'node-title'),
        array('data' => drupal_render($form['node'][$nid]['type']), 'class' => 'node-type'),
        array('data' => drupal_render($form['node'][$nid]['language']), 'class' => 'node-language'),
        array('data' => drupal_render($form['node'][$nid]['status']), 'class' => 'node-status'),
        array('data' => drupal_render($form['node'][$nid]['changed']), 'class' => 'node-changed'),
      );
      $rows[] = $row;
    }
  }

  $output = theme('table', $header, $rows, array('id' => 'xliff-bulk-export-node-list'));

  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }
  $output .= drupal_render($form);

  drupal_add_css(drupal_get_path('module', 'xliff_bulk') .'/css/xliff-bulk-export.css');
  return $output;
}

/**
 * Tar creation function. Written by dmitrig01.
 *
 * @param $files
 *   A keyed array where the key is the filepath and the value is the
 *   string contents of the file.
 *
 * @return
 *   A string of the tar file contents.
 */
function xliff_bulk_tar_create($files) {
  $tar = '';
  foreach ($files as $name => $contents) {
    $binary_data_first = pack("a100a8a8a8a12A12",
      $name,
      '100644 ', // File permissions
      '   765 ', // UID,
      '   765 ', // GID,
      sprintf("%11s ", decoct(strlen($contents))), // Filesize,
      sprintf("%11s", decoct(time())) // Creation time
    );
    $binary_data_last = pack("a1a100a6a2a32a32a8a8a155a12", '', '', '', '', '', '', '', '', '', '');

    $checksum = 0;
    for ($i = 0; $i < 148; $i++) {
      $checksum += ord(substr($binary_data_first, $i, 1));
    }
    for ($i = 148; $i < 156; $i++) {
      $checksum += ord(' ');
    }
    for ($i = 156, $j = 0; $i < 512; $i++, $j++) {
      $checksum += ord(substr($binary_data_last, $j, 1));
    }

    $tar .= $binary_data_first;
    $tar .= pack("a8", sprintf("%6s ", decoct($checksum)));
    $tar .= $binary_data_last;

    $buffer = str_split($contents, 512);
    foreach ($buffer as $item) {
      $tar .= pack("a512", $item);
    }
  }
  if (function_exists('gzencode')) {
    $tar = gzencode($tar);
  }
  return $tar;
}
